package org.nakedobjects.applib.clock;

import org.nakedobjects.applib.ApplicationException;

/**
 * Represents the current time.
 * 
 * <p>
 * The clock is used primarily by the temporal value classes, and is accessed by the
 * NOF as a singleton.  The actual implementation used can be configured at startup,
 * but once specified the clock instance cannot be changed.
 * 
 * <p>
 * Unless another {@link Clock} implementation has been installed, the first call to 
 * {@link #getInstance()} will instantiate an instance of {@link SystemClock}.
 * Alternate clocks can be created via suitable subclasses, but this must be done
 * <b><i>before</i></b> the first call to {@link #getInstance()}.
 */
public abstract class Clock {
    private static Clock instance;
    private static boolean isReplaceable = true;

    public final static Clock getInstance() {
        if (instance == null) {
            instance = new SystemClock();
            isReplaceable = false;
        }
        return instance;
    }

    public static long getTime() {
        return getInstance().time();
    }

    protected Clock() {
        if (!isReplaceable && instance != null) {
            throw new ApplicationException("Clock already set up");
        }
        instance = this;
    }
    
    /**
     * Gets the current time in milliseconds since midnight, January 1, 1970 UTC
     */
    protected abstract long time();
}

final class SystemClock extends Clock {
    protected long time() {
        return System.currentTimeMillis();
    }
}
// Copyright (c) Naked Objects Group Ltd.
