package org.nakedobjects.applib.security;

/**
 * Details, obtained from the container, about the user and his roles.
 * Read-only.
 */
public final class UserMemento {
	public static String fieldOrder() {
		return "name, roles";
	}

	private final String name;
	private final RoleMemento[] roles;

	/**
	 * Creates a new user with the specified name, but no assigned roles.
	 */
	public UserMemento(final String name) {
		this(name, new RoleMemento[0]);
	}

	/**
	 * Creates a new user with the specified name and assigned roles.
	 */
	public UserMemento(final String name, final RoleMemento[] roles) {
		if (name == null) {
			throw new IllegalArgumentException("Name not specified");
		}
		this.name = name;
		if (roles == null) {
			throw new IllegalArgumentException("Roles not specified");
		}
		this.roles = roles;
	}

	/**
	 * Name of the user - as provided by the container
	 */
	public String getName() {
		return name;
	}

	/**
	 * The set of roles this user part of.
	 */
	public RoleMemento[] getRoles() {
		return roles;
	}

	/**
	 * Determines if the user fulfills the specified role.
	 */
	public boolean hasRole(final RoleMemento role) {
		return hasRole(role.getName());
	}

	/**
	 * Determines if the user fulfills the specified role. Roles are compared
	 * lexically by role name.
	 */
	public boolean hasRole(final String role) {
		for (int i = 0; i < roles.length; i++) {
			if (roles[i].getName().equals(role)) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Determine if the specified user is this user. Returns true if the names
	 * match (is case sensitive).
	 */
	public boolean isCurrentUser(final String user) {
		if (user == null) {
			throw new IllegalStateException("no user specified");
		}
		return name.equals(user);
	}

	/**
	 * Title of the user is the name.
	 */
	public String titleString() {
		return name;
	}

	public String toString() {
		String rls = "";
		for (int i = 0; roles != null && i < roles.length; i++) {
			rls += roles[i].getName() + " ";
		}
		return "User [name=" + getName() + ",roles=" + rls + "]";
	}

}
// Copyright (c) Naked Objects Group Ltd.
