package org.nakedobjects.applib.value;

import java.math.BigDecimal;


public class Money extends Magnitude {
    private static final long serialVersionUID = 1L;
    private static final int[] cents = new int[] { 1, 10, 100, 100 };
    private final long amount;
    private final String currency;

    public Money(final double amount, final String currency) {
        assertCurrencySet(currency);
        this.currency = currency.toUpperCase();
        this.amount = Math.round(amount * centFactor());
    }

    public Money(final long amount, final String currency) {
        assertCurrencySet(currency);
        this.currency = currency.toUpperCase();
        this.amount = amount * centFactor();
    }
    
    private void assertCurrencySet(final String currency) {
        if (currency == null || currency.equals("")) {
            throw new IllegalArgumentException("Currency must be specified");
        }
        if (currency.length() != 3) {
            throw new IllegalArgumentException("Invalid currency code '" + currency + "'");
        }
    }

    /**
     * Add the specified money to this money.
     */
    public Money add(final Money money) {
        assertSameCurrency(money);
        return newMoney(amount + money.amount);
    }

    private void assertSameCurrency(Money money) {
        if (!money.getCurrency().equals(getCurrency())) {
            throw new IllegalArgumentException("Not the same currency: " + getCurrency() + " & " + money.getCurrency());
        }
    }

    private int centFactor() {
        return cents[getFractionalDigits()];
    }

    /**
     * Returns this value as a double.
     */
    public double doubleValue() {
        return amount / (double) centFactor();
    }

    public boolean equals(final Object obj) {
        if (this == obj) {
            return true;
        }
        if (!(obj instanceof Money)) {
            return false;
        }
        Money object = (Money) obj;
        return object.currency.equals(currency) && object.amount == amount;
    }

    /**
     * Returns this value as a float.
     */
    public float floatValue() {
        return (float) amount;
    }

    public BigDecimal getAmount() {
        return BigDecimal.valueOf(amount, getFractionalDigits());
    }

    public String getCurrency() {
        return currency;
    }

    private int getFractionalDigits() {
        return 2;
    }

    public boolean hasSameCurrency(final Money money) {
        return currency.equals(money.currency);
    }
    
    /**
     * Returns this value as an int.
     */
    public int intValue() {
        return (int) amount;
    }

    public boolean isEqualTo(final Magnitude magnitude) {
        if (magnitude instanceof Money && hasSameCurrency((Money) magnitude)) {
            return ((Money) magnitude).amount == amount;
        } else {
            throw new IllegalArgumentException("Parameter must be of type Money and have the same currency");
        }
    }

    public boolean isGreaterThanZero() {
        return amount > 0;
    }

    public boolean isLessThan(final Magnitude magnitude) {
        if (magnitude instanceof Money && hasSameCurrency((Money) magnitude)) {
            return amount < ((Money) magnitude).amount;
        } else {
            throw new IllegalArgumentException("Parameter must be of type Money and have the same currency");
        }
    }

    /**
     * Returns true if this value is less than zero.
     */
    public boolean isLessThanZero() {
        return amount < 0;
    }

    public boolean isZero() {
        return amount == 0;
    }

    /**
     * Returns this value as an long.
     */
    public long longValue() {
        return (long) amount;
    }

    private Money newMoney(final long amount) {
        return new Money(amount / (centFactor() * 1.0), this.currency);
    }

    /**
     * Subtract the specified amount from this value.
     */
    public Money subtract(final Money money) {
        assertSameCurrency(money);
        return newMoney(amount - money.amount);
    }
    
    public String toString() {
        return amount / (centFactor() * 1.0) + " " + currency;
    }
}
// Copyright (c) Naked Objects Group Ltd.
